@@ -4,7 +4,7 @@
 #  | .__/  \___| \___||_|\_\
 #  |_| like print, but easy.
 
-__version__ = "24.0.0"
+__version__ = "24.0.1"
 
 """
 See https://github.com/salabim/peek for details
@@ -42,6 +42,7 @@ if Pythonista:
     import clipboard
 else:
     import colorama
+    colorama.just_fix_windows_console()
     import pyperclip
 
 try:
@@ -67,28 +68,28 @@ colors = dict(
 colors["-"] = "\033[0m"
 
 
-ansi_to_hexa = {
-    "\033[0;30m": "#000000",
-    "\033[0;31m": "#FF0000",
-    "\033[0;32m": "#00FF00",
-    "\033[0;33m": "#FFFF00",
-    "\033[0;34m": "#0080FF",
-    "\033[0;35m": "#FF00FF",
-    "\033[0;36m": "#00FFFF",
-    "\033[0;37m": "#FFFFFF",
-    "\033[0m": "",
+ansi_to_rgb = {
+    "\033[0;30m": (0, 0, 0),
+    "\033[0;31m": (1, 0, 0),
+    "\033[0;32m": (0, 1, 0),
+    "\033[0;33m": (1, 1, 0),
+    "\033[0;34m": (0, 0.7, 1),
+    "\033[0;35m": (1, 0, 1),
+    "\033[0;36m": (0, 1, 1),
+    "\033[0;37m": (1, 1, 1),
+    "\033[0m": (),
 }
 
 
-def dealias(name):
+def de_alias(name):
     return alias_name.get(name, name)
 
 
-def check_validity(name, value, message=""):
+def check_validity(name, value):
     name_org = name
-    name = dealias(name)
+    name = de_alias(name)
     if name not in name_default:
-        raise ValueError(f"parameter {name} not allowed{message}")
+        raise AttributeError(f"attribute {name} not allowed{in_read_toml_message}")
 
     if value is None:
         return
@@ -102,7 +103,7 @@ def check_validity(name, value, message=""):
             return
         except Exception:
             pass
-        raise TypeError("output should be a callable, str, Path or open text file.")
+        raise AttributeError("output should be a callable, str, Path or open text file.")
 
     if name == "serialize":
         if callable(value):
@@ -114,6 +115,10 @@ def check_validity(name, value, message=""):
         if isinstance(value, str) and value.lower() in colors:
             return
 
+        elif name == "delta":
+            if isinstance(value, numbers.Number):
+                return
+
     elif name == "enforce_line_length":
         if value:
             if isinstance(value, numbers.Number) and value > 0:
@@ -152,7 +157,7 @@ def check_validity(name, value, message=""):
     else:
         return
 
-    raise ValueError(f"incorrect {name_org}: {value}{message}")
+    raise AttributeError(f"incorrect {name_org}: {repr(value)}{in_read_toml_message}")
 
 
 class Source(executing.Source):
@@ -173,24 +178,31 @@ def change_path(new_path):  # used in tests
 _fixed_perf_counter = None
 
 
-def read_toml():
-    this_path = Path(".").resolve()
+def spec_to_attributes(d):
+    result = {}
+    for name, value in d.items():
+        check_validity(name, value)
+        name = de_alias(name)
+        if name == "delta" and value is not None:
+            result["perf_counter"] = perf_counter()
+        result[name] = value
+    return result
 
+
+def read_toml():
+    global in_read_toml_message
+    this_path = Path(".").resolve()
     for i in range(len(this_path.parts), 0, -1):
         toml_file = Path(this_path.parts[0]).joinpath(*this_path.parts[1:i], "peek.toml")
         if toml_file.is_file():
+            in_read_toml_message = f" in reading {toml_file}"
             with open(toml_file, "r") as f:
                 config_as_str = f.read()
-            config = tomlib.loads(config_as_str)
-            for name, value in config.items():
-                check_validity(name, value, message=f" while processing {toml_file}")
-            return config
-            break
+            return tomlib.loads(config_as_str)
     return {}
 
 
 def no_source_error():
-
     raise NotImplementedError(
         """
 Failed to access the underlying source code for analysis. Possible causes:
@@ -212,28 +224,36 @@ def return_args(args, return_none):
 
 class _Peek:
     def __init__(self, parent=None, **kwargs):
-        self._attributes = {}
-        for name, value in kwargs.items():
-            check_validity(name, value)
-            setattr(self, name, value)
+        self._attributes = spec_to_attributes(kwargs)
         self._parent = parent
 
-    def __repr__(self):
-        pairs = [str(name) + "=" + repr(getattr(self, name)) for name in name_default if name != "serialize"]
-        return "peek.new(" + ", ".join(pairs) + ")"
+    def new(self, ignore_toml=False, **kwargs):
+        if ignore_toml:
+            return _Peek(**kwargs, parent=peek_no_toml)
+        else:
+            return _Peek(**kwargs, parent=peek_toml)
 
-    def fix_perf_counter(self, val):  # for tests
-        global _fixed_perf_counter
-        _fixed_perf_counter = val
+    def fork(self, **kwargs):
+        return _Peek(**kwargs, parent=self)
 
-    def __getattr__(self, item):
-        item = dealias(item)
-        if item in name_default:
+    def clone(self, **kwargs):
+        clone = _Peek(parent=self._parent)
+        clone._attributes = {**self._attributes, **spec_to_attributes(kwargs)}
+        return clone
+
+    def configure(self, **kwargs):
+        self._attributes.update(spec_to_attributes(kwargs))
+
+    def __getattr__(self, item, spec=False):
+        item = de_alias(item)
+        if item in name_default or item == "perf_counter":
             node = self
             while not item in node._attributes or node._attributes[item] is None:
                 node = node._parent
             if item == "delta":
-                return perf_counter() - node._attributes[item]
+                return perf_counter() - node._attributes["perf_counter"] + node._attributes["delta"]
+            elif item == "perf_counter":
+                return node._attributes["delta"]
             else:
                 return node._attributes[item]
         else:
@@ -242,18 +262,23 @@ class _Peek:
     def __setattr__(self, item, value):
         if item in ("_parent", "is_context_manager", "line_number_with_filename_and_parent", "save_traceback", "enter_time", "as_str", "_attributes"):
             return super().__setattr__(item, value)
+        self._attributes.update(spec_to_attributes({item: value}))
 
-        if item not in name_and_alias_default:
-            raise AttributeError(f"attribute {item} not allowed")
+    def __repr__(self):
+        pairs = [
+            str(name) + "=" + repr(getattr(self, "perf_counter") if name == "delta" else getattr(self, name)) for name in name_default if name != "serialize"
+        ]
+        return "peek.new(" + ", ".join(pairs) + ")"
 
-        check_validity(item, value)
-        item = dealias(item)
+    def __str__(self):
+        pairs = [
+            str(name) + "=" + repr(getattr(self, "perf_counter") if name == "delta" else getattr(self, name)) for name in name_default if name != "serialize"
+        ]
+        return "peek with attributes:\n    " + "\n    ".join(pairs) + ")"
 
-        if item == "delta" and value is not None:
-            self._attributes[item] = perf_counter() - value
-
-        else:
-            self._attributes[item] = value
+    def fix_perf_counter(self, val):  # for tests
+        global _fixed_perf_counter
+        _fixed_perf_counter = val
 
     def do_show(self):
         if self.filter.strip() != "":
@@ -261,14 +286,6 @@ class _Peek:
                 return False
         return self.enabled
 
-    def copy_to_clipboard(self, value, confirm=True):
-        if Pythonista:
-            clipboard.set(str(value))
-        else:
-            pyperclip.copy(str(value))
-        if confirm:
-            print(f"copied to clipboard: {value}")
-
     def __call__(self, *args, as_str=False, **kwargs):
         codes = {}
         this = self.fork(**kwargs)
@@ -507,28 +524,6 @@ class _Peek:
 
         return return_args(args, this.return_none)
 
-    def configure(self, **kwargs):
-        for name, value in kwargs.items():
-            check_validity(name, value)
-            self._attributes[name] = value
-        return self
-
-    def new(self, ignore_toml=False, **kwargs):
-        if ignore_toml:
-            return _Peek(**kwargs, parent=peek_no_toml)
-        else:
-            return _Peek(**kwargs, parent=peek_toml)
-
-    def fork(self, **kwargs):
-        return _Peek(**kwargs, parent=self)
-
-    def clone(self, **kwargs):
-        clone = _Peek(parent=self._parent)
-        clone._attributes=self._attributes.copy()
-        for name,value in kwargs:
-            setattr(clone,name,value)    
-        return clone
-
     @contextlib.contextmanager
     def preserve(self):
         save = dict(self._attributes)
@@ -567,7 +562,7 @@ class _Peek:
         if not omit_context_separator and context:
             context += self.context_separator
 
-        return (self.prefix() if callable(self.prefix) else self.prefix) + context
+        return str(self.prefix() if callable(self.prefix) else self.prefix) + context
 
     def add_color_value(self, s):
         if self.output != "stdout" or self.as_str:
@@ -590,17 +585,13 @@ class _Peek:
                     s = colors[self.color.lower()] + s + colors["-"]
                     if Pythonista:
                         while s:
-                            for ansi, hexa in ansi_to_hexa.items():
+                            for ansi, rgb in ansi_to_rgb.items():
                                 if s.startswith(ansi):
-                                    if hexa == "":
-                                        console.set_color()
-                                    else:
-                                        rgb = tuple(int(hexa[i : i + 2], 16) / 255 for i in range(1, 7, 2))
-                                        console.set_color(*rgb)
+                                    console.set_color(*rgb)
                                     s = s[len(ansi) :]
                                     break
                             else:
-                                print(s[0], end="", file=sys.stdout)
+                                print(s[0], end="")
                                 s = s[1:]
                         print()
                     else:
@@ -630,6 +621,14 @@ class _Peek:
             else:
                 print(s, file=self.output)
 
+    def copy_to_clipboard(self, value, confirm=True):
+        if Pythonista:
+            clipboard.set(str(value))
+        else:
+            pyperclip.copy(str(value))
+        if confirm:
+            print(f"copied to clipboard: {value}")
+
     def traceback(self):
         if self.show_traceback:
             if isinstance(self.wrap_indent, numbers.Number):
@@ -660,11 +659,12 @@ class _Peek:
         return self.add_color_value(self.serialize(obj, **kwargs).replace("\\n", "\n"))
 
 
+store_perf_counter = perf_counter()
 name_alias_default = (
     ("color", "col", "-"),
-    ("color_value", "colv", ""),
-    ("context_separator", "cs", " ==> "),
+    ("color_value", "col_val", ""),
     ("compact", "", False),
+    ("context_separator", "cs", " ==> "),
     ("delta", "", 0),
     ("depth", "", 1000000),
     ("enabled", "", True),
@@ -689,26 +689,22 @@ name_alias_default = (
     ("sort_dicts", "", False),
     ("to_clipboard", "clip", False),
     ("underscore_numbers", "un", False),
-    ("wrap_indent", "", "    "),
     ("values_only", "vo", False),
     ("values_only_for_fstrings", "voff", False),
+    ("wrap_indent", "", "    "),
 )
-
 alias_name = {alias: name for (name, alias, default) in name_alias_default if alias}
 name_default = {name: default for (name, alias, default) in name_alias_default}
 alias_default = {alias: default for (name, alias, default) in name_alias_default if alias}
 name_and_alias_default = {**name_default, **alias_default}
 
+in_read_toml_message = ""
 peek_no_toml = _Peek(**name_default)
-peek_toml = peek_no_toml.fork(**read_toml())
-#_Peek(parent=peek_no_toml)
-# for name, value in read_toml().items():
-#     setattr(peek_toml,name,value)
-
-peek=peek_toml.new()
+peek_toml = _Peek(**{**name_default, **read_toml()})
+in_read_toml_message = ""
+peek = peek_toml.new()
 builtins.peek = peek
 
-
 class PeekModule(types.ModuleType):
     def __call__(self, *args, **kwargs):
         return peek(*args, **kwargs)
@@ -719,6 +715,12 @@ class PeekModule(types.ModuleType):
     def __getattr__(self, item):
         return getattr(peek, item)
 
+    def __repr__(self):
+        return repr(peek)
+
+    def __str__(self):
+        return str(peek)
+
 
 if __name__ != "__main__":
     sys.modules["peek"].__class__ = PeekModule
